package kartelj.aleksandar.various;

import java.util.List;

import org.apache.commons.collections.ComparatorUtils;
import org.apache.spark.SparkConf;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.api.java.JavaSparkContext;

/**
 * http://spark.apache.org/docs/latest/programming-guide.html
 *
 */
public class Prvi {
	public static void main(String[] args) {
		SparkConf conf = new SparkConf();
		conf.setAppName("app");
		conf.setMaster("local[*]");

		JavaSparkContext sc = new JavaSparkContext(conf);
		JavaRDD<String> lines = sc.textFile("src/brojevi1.txt");
		// prvo smo niz stringova JavaRDD<String> koji zapravo predstavljaju
		// brojeve mapirali i prebaciili
		// u JavaRDD<Integer> i onda smo u prirodnom uredjenju uzeli prvih 100
		// da nismo prebacili u brojeve vrsilo bi se leksikografsko poredjenje
		//List<Integer> top100 = lines.map(x -> Integer.valueOf(x)).takeOrdered(
		//		100, new DescendingComparator<Integer>());
		// Koristimo nas komparator jer onaj ugradjeni nije serializable sto
				// znaci
				// da taj ugradjeni ne implementira interfejs Serializable koji mu
				// garantuje
				// da objekat comparator moze da se zapakuje kao datoteka i posalje kroz
				// mrezu
				// prema nekom od cvorova distribuiranog sistema
		List<Integer> top100 = lines.map(x -> Integer.valueOf(x)).takeOrdered(
				100, new GenericSerializableComparator<Integer>((x, y) -> y - x));
		// Ovo je jos bolja verzija comparatora jer je potpuno genericki, prosledjujemo mu 
		// lambda izraz poredjenja kroz konstruktor
		System.out.println(top100);
	}
}
